/**************************************************************************************************/

/*
 * File: list.c
 * Author: Roman Lysecky
 * Date Created: 18 September 2012
 *
 * Description: Definition of a set of functions for manipulating a list of dynamically
 * allocated strings.
 *
 * Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2012.
 *
 */

/**************************************************************************************************/

#include "bool.h"
#include "listelmt.h"
#include "list.h"
#include <stdlib.h>
#include <stdio.h>

/**************************************************************************************************/

/* initialize list to empty list */
void
list_init(List* list)
{
   if( list == NULL ) return;
   
   list->head = NULL;
   list->tail = NULL;
   list->size = 0;
}

/**************************************************************************************************/

/*
 * destroys the list by freeing all memory allocated for each line element returning the list
 * to an empty list
 */
void
list_destroy(List * list)
{
   if( list == NULL ) return;
   
   /* while the list of line elements is not empty, remove the element at the head of the list */
   while (list->size != 0) {
      list_rem_next(list, NULL);
      
   }
}

/**************************************************************************************************/

/*
 * inserts new string into list after the given list element
 * location. If the list element location is NULL, inserts line element at head of list
 */
int
list_ins_next(List *list, ListElmt *element, char *str)
{
   ListElmt *new_element;
   if( list == NULL ) return -1;
   
   new_element = (ListElmt*)malloc(sizeof(ListElmt));
   if( new_element== NULL ) return -1;
   
   new_element->str = str;
   
   /* insertion at head of list */
   if( element == NULL ) {
      
      new_element->next = list->head;
      list->head = new_element;
      if(list->size == 0 ) list->tail = new_element;      
   }
   /* insertion somewhere other than head */
   else {
      if( element->next == NULL ) list->tail = new_element;
      new_element->next = element->next;
      element->next = new_element;
   }
   list->size++;
   
   return 0;
}

/**************************************************************************************************/

/*
 * removes the  element after the the given list element location. If the list element location
 * is NULL, removes the head of the list
 */
int
list_rem_next(List *list, ListElmt *element)
{
   ListElmt *old_element;
   
   if( list == NULL || list->size == 0 ) return -1;
   
   /* removal from head of list */
   if( element == NULL ) {
      old_element = list->head;
      list->head = list->head->next;
      
      // if list will become empty, set tail to NULL
      if( list->size == 1 ) list->tail = NULL;
   }
   /* removal from somewhere otehr than the head */
   else {
      /* cannot remove next element if it doesn't exist */
      if( element->next == NULL ) return -1;
      
      old_element = element->next;
      element->next = element->next->next;
      if( element->next == NULL ) list->tail = element;
   }
   
   free(old_element->str);
   free(old_element);
   list->size--;
   return 0;
}

/**************************************************************************************************/

/* returns a pointer to the element at the head of the list */
ListElmt *
list_head(List *list)
{
   if( list == NULL ) return NULL;
   return list->head;
}

/**************************************************************************************************/

/* returns the number of elements within the list */
int
list_size(List *list)
{
   if( list == NULL ) return -1;
   return list->size;
}

/**************************************************************************************************/

/* returns pointer to next line element within the list */
ListElmt *
list_next(ListElmt *element)
{
   if( element == NULL ) return NULL;
   return element->next;
}


/**************************************************************************************************/

/* prints list of line elements within list */
void
list_display(List *list)
{
   ListElmt *element;
   
   if( list == NULL ) return;
   
   element = list_head(list);
   while(element!=NULL) {
      element = list_next(element);
   }
}

/**************************************************************************************************/

